/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


import com.novell.nds.dirxml.driver.jdbc.util.JDBCLib;
import com.novell.nds.dirxml.driver.jdbc.util.def.JDBCConstants;

import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;


/**
 *  An abstract directory time value.  Represents either a 32-bit
 *  integer string (e.g., "1013368206") or a canonical timestamp, date, or time string
 *  (i.e., <code>CCYYMMDDHHMMSSNNNNNNNNN</code>, <code>CCYYMMDD</code> or <code>HHMMSS</code>).
 */
public
class DSTime
    extends AbstractTimeValue
{

    static private
    long
    ToMillis(long seconds)
    {
        return (seconds * 1000);
    }//ToMillis(long)


    static private
    int
    GetNanosAsInt(String snanos)
    {
        //make 9-digit integer from less-than-9-digit string
        StringBuffer buffer;

        buffer = new StringBuffer(MAX_SCALE_NANOS);
        buffer.append(snanos);
        for(int i=(MAX_SCALE_NANOS-snanos.length()); i>0; i--)
        {
            buffer.append(0);
        }
        return Integer.parseInt(buffer.toString());
    }//GetNanosAsInt()


    static private
    Timestamp
    ParseTimestampFromLong(String lng)
    {
        //NDS precision in seconds; Database precision in
        //  milliseconds
        return new Timestamp(ToMillis(Long.parseLong(lng)));
    }//ParseTimestampFromLong()


    static private
    Timestamp
    ParseTimestampFromString(String string)
        throws IllegalArgumentException,
               ParseException
    {
        //try to parse string format:  CCYYMMDDHHMMSSNNNNNNNNN
        //long has max 19-digit precision (base 10); the above
        //  string format contains 23 digits, hence the
        //  IllegalArgumentException

        SimpleDateFormat sdf;
        Timestamp ts;
        String nonNanoPart, nanoPart;

        if(string.length() < DS_LEN_TIMESTAMP_MIN)
        {
            throw new IllegalArgumentException
                ("Timestamp is too short.  String timestamps (e.g., 'CCYYMMDDHHMMSSNNNNNNNNN') must be at least " +
                 DS_LEN_TIMESTAMP_MIN + " characters long (i.e., 'CCYYMMDDHHMMSS').");
        }

        //take first 14 digits as non-nanos
        nonNanoPart = string.substring(JDBCConstants.JAVA_INDEX_1ST,
                                       DS_LEN_TIMESTAMP_MIN);
        sdf = new SimpleDateFormat(FORMAT_TIMESTAMP);
        ts = new Timestamp(sdf.parse(nonNanoPart).getTime());
        if(string.length() > DS_LEN_TIMESTAMP_MIN)
        {
            //take last 9 digits as nanos; truncate if too long; not sure what will happen if the
            //  beginning index is same as length
            nanoPart = string.substring(DS_LEN_TIMESTAMP_MIN,
                                        Math.min(LEN_STRING_TIMESTAMP, string.length()));
            ts.setNanos(GetNanosAsInt(nanoPart));
        }//if
        else
        {
            ts.setNanos(0); //no nanos
        }

        return ts;
    }//ParseTimestampFromString()


    static private
    Date
    ParseDateFromLong(String lng)
    {
        //NDS precision in seconds; Database precision in
        //  milliseconds
        return new Date(JDBCLib.ToMillis(Long.parseLong(lng)));
    }//ParseDateFromLong()


    static private
    Time
    ParseTimeFromLong(String lng)
    {
        //NDS precision in seconds; Database precision in
        //  milliseconds
        return new Time(JDBCLib.ToMillis(Long.parseLong(lng)));
    }//parseTimeFromLong()


    static private
    Date
    ParseDateFromString(String string)
        throws IllegalArgumentException,
               ParseException
    {
        //try to parse string format:  CCYYMMDD

        SimpleDateFormat sdf;
        java.sql.Date date;

        if(string.length() < LEN_STRING_DATE)
        {
            throw new IllegalArgumentException
                ("Date value is too short.  String dates (e.g., 'CCYYMMDD') must be " +
                 LEN_STRING_DATE + " characters long.");
        }

        sdf = new SimpleDateFormat(FORMAT_DATE);
        date = new Date(sdf.parse(string).getTime());

        return date;
    }//ParseDateFromString()


    static private
    Time
    ParseTimeFromString(String string)
        throws IllegalArgumentException,
               ParseException
    {
        //try to parse string format:  HHMMSS

        SimpleDateFormat sdf;
        Time time;

        if(string.length() < LEN_STRING_TIME)
        {
            throw new IllegalArgumentException
                ("Time value is too short.  String times (e.g., 'HHMMSS') must be " +
                 LEN_STRING_TIME + " characters long.");
        }

        sdf = new SimpleDateFormat(FORMAT_TIME);
        time = new Time(sdf.parse(string).getTime());

        return time;
    }//ParseTimeFromString()


    protected java.util.Date _instance;


    //no default constructor is permitted
    private
    DSTime(){};


    /**
     * Instantiates a directory time value.
     *
     * @param dsTime may be <code>null</code>;
     * format should be <code>CCYYMMDDHHMMSSNNNNNNNNN</code>,
     * <code>CCYYMMDD</code>, <code>HHMMSS</code> or a 32-bit integer
     * value
     * @param sqlClass indicates the type of object to be returned
     * from the <code>getInstance():java.util.Date</code> method;
     * should be <code>java.sql.Timestamp</code>,
     * <code>java.sql.Date</code> or <code>java.sql.Time</code>;
     * the default value is <code>java.sql.Timestamp</code>
     * @param inFormat should be "<code>1</code>" (integer) or
     * "<code>2</code>" (string); the
     * default value is "<code>1</code>" (integer);
     * may be <code>null</code>
     */
    public
    DSTime
    (
        String dsTime,
        String sqlClass,
        String inFormat
    )
    {
        impl(dsTime, sqlClass, inFormat);
    }//DSTime()


    private
    void
    impl
    (
        String dsTime,
        String sqlClass,
        String inFormat
    )
    {
        _time = dsTime;

        if(DBLib.IsNotValue(dsTime))
        {
            _parseError = true;
            _errorMessage = "Null or empty string is not a valid directory time value.";
            addException(new IllegalArgumentException(_errorMessage));
        }
        else
        {
            dsTime = dsTime.trim();
            setClass(sqlClass);
            setOutFormat(inFormat);

            try
            {
                setDSTime(dsTime);
            }
            catch(ParseException p)
            {
                _parseError = true;
                _errorMessage = "Malformed directory time value '" + dsTime +
                                "'.  " + p.getMessage();
                addException(p);
            }
            catch(IllegalArgumentException iae)
            {
                _parseError = true;
                _errorMessage = "Invalid directory time value '" + dsTime +
                                "'.  " + iae.getMessage();
                addException(iae);
            }
        }//else
    }//impl()


    protected
    void
    setDSTime(String dsTime)
      throws ParseException, IllegalArgumentException
    {
        if(_outFormat == DSTimeFormat.INT)
        {
            //parse a long rather than an int to preserve backward compatibility w/ version 1.6
            parseLong(dsTime);
        }//if
        else if(_outFormat == DSTimeFormat.STRING)
        {
            //could be Java or canonical string
            if((dsTime.indexOf(":") < 0) &&
               (dsTime.indexOf("-") < 0))
            {
                //no formatting chars indicates canonical string
                //e.g., CCYYMMDDHHMMSSNNNNNNNNN, CCYYMMDD and HHMMSS

                parseCanonicalString(dsTime);
            }//if
            else
            {
                //formatting chars indicate Java string
                //e.g., yyyy-mm-dd hh:mm:ss.fffffffff, yyyy-mm-dd and hh:mm:ss

                parseJavaString(dsTime);
            }//else
        }//else if
        else if(_outFormat == DSTimeFormat.JAVA)
        {
            parseJavaString(dsTime);
        }
        else
        {
            throw new RuntimeException
                (Text.BROKEN_IF_ELSE);
        }
    }//setDSTime()


    private
    void
    parseLong(String dsTime)
      throws IllegalArgumentException
    {
        if(_class == java.sql.Timestamp.class)
        {
            _instance = ParseTimestampFromLong(dsTime);
        }
        else if(_class == java.sql.Date.class)
        {
            _instance = ParseDateFromLong(dsTime);
        }
        else if(_class == java.sql.Time.class)
        {
            _instance = ParseTimeFromLong(dsTime);
        }
        else
        {
            throw new RuntimeException
                (Text.BROKEN_IF_ELSE);
        }
    }//parseLong()


    private
    void
    parseCanonicalString(String dsTime)
      throws ParseException, IllegalArgumentException
    {
        if(_class == java.sql.Timestamp.class)
        {
            _instance = ParseTimestampFromString(dsTime);
        }
        else if(_class == java.sql.Date.class)
        {
            _instance = ParseDateFromString(dsTime);
        }
        else if(_class == java.sql.Time.class)
        {
            _instance = ParseTimeFromString(dsTime);
        }
        else
        {
            throw new RuntimeException
                (Text.BROKEN_IF_ELSE);
        }
    }//parseCanoncialString()


    private
    void
    parseJavaString(String dsTime)
      throws ParseException, IllegalArgumentException
    {
        if(_class == java.sql.Timestamp.class)
        {
            _instance = Timestamp.valueOf(dsTime);
        }
        else if(_class == java.sql.Date.class)
        {
            _instance = Date.valueOf(dsTime);
        }
        else if(_class == java.sql.Time.class)
        {
            _instance = Time.valueOf(dsTime);
        }
        else
        {
            throw new RuntimeException
                (Text.BROKEN_IF_ELSE);
        }
    }//parseJavaString()


    /**
     * Returns the string representation of a <code>java.sql.Timestamp</code>,
     * <code>java.sql.Date</code> or <code>java.sql.Time</code> instance.
     *
     * @return may return empty string in the event of a parse error;
     * will not return <code>null</code>
     */
    public
    String
    getDBTime()
    {
       return (_instance == null) ? _time : _instance.toString();
    }


    /**
     * Returns an instance of <code>java.sql.Timestamp</code>,
     * <code>java.sql.Date</code> or <code>java.sql.Time</code> instance.
     *
     * @return may return <code>null</code> in the event of a parse error
     */
    public
    Object
    getInstance()
    {
        return _instance;
    }

}//class DSTimestamp
