#ifndef _UCSFUNC_H_
#define	_UCSFUNC_H_
/*******************************************************************
* Name: ucsfunc.h
* Description: Utility functions for dealing with ucs characters, in
* particular with UTF-16 encoded characters strings
* Tabs: 4
*
* Copyright (C) 2000 Novell, Inc., All Rights Reserved
* COMPANY CONFIDENTIAL -- NOT FOR DISTRIBUTION
*
* %version:   2 %
* %created_by:   pblanchard %
* %date_modified:  Wed Feb 06 11:44:08 2008 %
*******************************************************************/
#include	<stddef.h>	//for size_t
//platform-specific support
#include	"plat_ucsfunc.h"

//define our UTF-16 type
typedef	unsigned short	unicode;

//hack placed here to avoid creating another file
#if	defined(_WINDOWS)
#define	stricmp	_stricmp
#endif

#if	defined(__cplusplus)
extern "C" {
#endif

//=============================================================================
//The following functions must be supported either by mapping them via 
//plat_ucsfunc.h to platform-supplied functions (like the wcs* functions on
//Win32) or by supplying definitions in platform-specific code.
//=============================================================================

#if	!defined(DEF_UTF16_LEN)
//++
//=============================================================================
// Method:	utf16_len
// Desc:	Return the length, in characters, of a null-terminated UTF-16
//			string.
// 
// Notes:
//			This is equivalent to strlen().
//			This must count UTF-16 surrogate pairs as two characters.
//=============================================================================
extern size_t					//number of 16-bit character entities
utf16_len(
	const unicode * ustr);		//null-terminated UTF-16 string
//--
#endif	//#if	!defined(DEF_UTF16_LEN)

#if	!defined(DEF_UTF16_CMP)
//++
//=============================================================================
// Method:	utf16_cmp
// Desc:	Compare UTF-16 strings according to code point values.
// 
// Notes:
//			This is equivalent to strcmp().
//			Return:
//				< 0 ustr1 less than ustr2
//				  0 ustr1 equal to ustr2
//				> 0 ustr1 greater than ustr2
//=============================================================================
extern int						//comparison result
utf16_cmp(
	const unicode * ustr1,		//null-terminated UTF-16 string
	const unicode * ustr2);		//null-terminated UTF-16 string
//--
#endif	//#if	!defined(DEF_UTF16_CMP)

#if	!defined(DEF_UTF16_CPY)
//++
//=============================================================================
// Method:	utf16_cpy
// Desc:	Copy a null-terminated UTF-16 string into a buffer.
// 
// Notes:
//			This is equivalent to strcpy().
//=============================================================================
extern unicode *				//pointer to dest
utf16_cpy(
	unicode * dest,				//pointer to buffer into which to copy
	const unicode * src);		//null-terminated UTF-16 string
//--
#endif	//#if	!defined(DEF_UTF16_CPY)

#if	!defined(DEF_UTF16_CAT)
//++
//=============================================================================
// Method:	utf16_cat
// Desc:	Concatenate a null-terminated UTF-16 string onto another
//			null-terminated UTF-16 string.
// 
// Notes:
//			This is equivalent to strcat().
//=============================================================================
extern unicode *				//pointer to dest
utf16_cat(
	unicode * dest,				//pointer to buffer containing a null-terminated UTF-16 string
	const unicode * src);		//null-terminated UTF-16 string to copy
//--
#endif	//#if	!defined(DEF_UTF16_CAT)

#if	!defined(DEF_UTF16_NCPY)
//++
//=============================================================================
// Method:	utf16_ncpy
// Desc:	Copy at most n UTF-16 characters into a buffer.
// 
// Notes:
//			This is equivalent to strncpy().
//			If count is <= the length of src a terminating null is not
//			automatically appended. If count > length of src then dest is 
//			padded with nulls up to length count.
//=============================================================================
extern unicode *				//pointer to dest
utf16_ncpy(
	unicode * dest,				//buffer into which to copy
	const unicode * src,		//null-terminated UTF-16 string
	size_t count);				//maximum number of characters to copy
//--
#endif	//#if	!defined(DEF_UTF16_NCPY)

#if	!defined(DEF_UTF16_TOSTR)
//++
//=============================================================================
// Method:	utf16_tostr
// Desc:	Convert a UTF-16 string into a char string, using the default
//			platform char encoding.
// 
// Notes:
//			On Windows, this is equivalent to wcstombs().
//		
//			Returns -1 cast to size_t on error.
//=============================================================================
extern size_t				//number of bytes written into str, not including terminating null
utf16_tostr(
	char * str,				//buffer into which to place char string.
	const unicode * src,	//null-terminated UTF-16 string.
	size_t count);			//size of str, in bytes
//--
#endif	//#if	!defined(DEF_UTF16_TOSTR)

#if	defined(__cplusplus)
}
#endif

#endif	//#ifndef _UCSFUNC_H_
