/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db.lock;


import java.util.Collections;
import java.util.LinkedList;
import java.util.List;


/**
 * A mutable database resource lock.  Consists of zero or more database-native SQL
 * statements that lock a database resource.
 */
public
class DBLock
{

    protected List _stmts;


    /**
     * Instantiates a database lock of zero statements.
     */
    public DBLock()
    {
        _stmts = Collections.EMPTY_LIST;
    }


    /**
     * Adds a statement to this lock.
     *
     * @param stmt may be <code>null</code>
     */
    public
    void
    addLockStatement(DBLockStatement stmt)
    {
        if(_stmts == Collections.EMPTY_LIST)
        {
            _stmts = new LinkedList();
        }

        _stmts.add(stmt);
    }


    /**
     * Retruns the SQL statements that comprise this lock.
     *
     * @return a list of zero or more <code>DBLockStatement</code>
     * objects; will not return <code>null</code>
     */
    public
    List
    getLockStatements()
    {
        return _stmts;
    }


    /**
     * Method indicating to the caller whether this lock's statements
     * should replace the ones that generated them.  Some of
     * the locking statements that might be used prior to a
     * SELECT statement, for instance, are effectively
     * equivalent to the original.
     * <p>
     * This method is currently not invoked in any rt.
     * Its use is reserved for future use.
     *
     * @return <code>true</code> if this lock's statements should
     * be executed instead of the statements used to generate
     * them; <code>false</code> otherwise
     */
    public
    boolean
    shouldSubstitute()
    {
        return false;
    }

}//class DBLock
