/*************************************************************************
Copyright (C) 1999-2008 Novell, Inc. All Rights Reserved. 

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND 
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE
AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK) THAT CONTAINS
THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS
TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S
SAMPLE CODE IN ITS PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION
RIGHTS TO MARKET, DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT
OF DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR
DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.

***************************************************************************/
/*******************************************************************
* Name: dllmain.cpp
* Description: Win32-specific file for CSkeletonDriver
* Tabs: 4
*******************************************************************/

#define	WIN32_LEAN_AND_MEAN
#include	<windows.h>
#include	"CSkeletonDriver.h"
#include	"SkeletonPublisher.h"

static bool newConsole = false;

static const char * MSG_DRIVER_TITLE = "CSkeletonDriver";

//++
//=========================================================================
// Purpose:
//		entry point for DLL
// Notes:
//=========================================================================
BOOL APIENTRY DllMain(
	HANDLE hModule, 
	DWORD  ul_reason_for_call, 
	LPVOID lpReserved)
//--
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

//++
//=========================================================================
// Purpose:
//		DirXML entry point for driver.
// Notes:
//		DirXML calls this function to get a driver instance.
//		This is the only function that MUST be exported from the module,
//		subject to platform considerations.
//=========================================================================
extern "C" CSKELETONDRIVER_API DriverShim * METHOD_CALL
CreateDriver(void)
//--
{
	return new CSkeletonDriver();
}

//++
//=========================================================================
// Purpose:
//		Platform-specific code to create a semaphore.
// Notes:
//		Support for SkeletonPublisher
//=========================================================================
SEMAPHORE_HANDLE
SkeletonPublisher::createSemaphore()
//--
{
	return	(SEMAPHORE_HANDLE)CreateSemaphore(
					NULL,			//security attributes
					0,				//not signalled initially
					1,				//max count
					NULL);			//name
}

//++
//=========================================================================
// Purpose:
//		Platform-specific code to wait on a semaphore (with timeout)
// Notes:
//		Support for SkeletonPublisher
//=========================================================================
void
SkeletonPublisher::waitSemaphore(
	SEMAPHORE_HANDLE semaphore,
	int timeoutInMilliseconds)
//--
{
	WaitForSingleObject((HANDLE)semaphore,timeoutInMilliseconds);
}

//++
//=========================================================================
// Purpose:
//		Platform-specific code to signal a semaphore
// Notes:
//		Support for SkeletonPublisher
//=========================================================================
void
SkeletonPublisher::signalSemaphore(
	SEMAPHORE_HANDLE semaphore)
//--
{
	ReleaseSemaphore((HANDLE)semaphore,1,NULL);
}

//++
//=========================================================================
// Purpose:
//		Platform-specific code to destroy a semaphore created by
//		SkeletonPublisher::createSemaphore()
// Notes:
//		Support for SkeletonPublisher
//=========================================================================
void
SkeletonPublisher::destroySemaphore(
	SEMAPHORE_HANDLE semaphore)
//--
{
	CloseHandle((HANDLE)semaphore);
}
